<?php

declare(strict_types = 1);

namespace Drupal\group_hidden_role\Entity\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\group\Entity\Form\GroupRelationshipForm;
use Drupal\group\Entity\GroupMembershipInterface;
use Drupal\group\Entity\GroupRelationshipInterface;

/**
 * Form handler for group relationship edit forms.
 */
class GroupRelationshipEditForm extends GroupRelationshipForm {

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $group_relationship = $form_state->getFormObject()->getEntity();
    assert($group_relationship instanceof GroupRelationshipInterface);

    if ($group_relationship instanceof GroupMembershipInterface) {
      $submitted_roles = $form_state->getValue('group_roles');
      foreach ($group_relationship->getRoles(FALSE) as $role) {
        if (
          $role->getThirdPartySetting('group_hidden_role', 'hidden_role') &&
          !in_array($role->id(), array_column($submitted_roles, 'target_id'))
        ) {
          // If the group membership has hidden roles that were not submitted
          // with the form, add them to the submitted roles.
          $submitted_roles[] = ['target_id' => $role->id()];
        }
      }
      $form_state->setValue('group_roles', $submitted_roles);
    }

    parent::submitForm($form, $form_state);
  }

}
